@@ -22,15 +22,13 @@
 
 #ifndef _CGIUTILS_H
 #define _CGIUTILS_H
-
+#include "compat.h"
 #include "config.h"
 #include "logging.h"
 #include "objects.h"
 #include "cgiauth.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+NAGIOS_BEGIN_DECL
 
 	/**************************** CGI REFRESH RATE ******************************/
 
@@ -81,6 +79,8 @@ extern "C" {
 #define MINISTATUS_CSS          "ministatus.css"
 #define SUMMARY_CSS             "summary.css"
 
+	/********************************* JAVASCRIPT INCLUDES **********************/
+#define JQUERY_JS		"jquery-1.7.1.min.js"	
 
 	/********************************* ICONS ************************************/
 
@@ -180,6 +180,10 @@ extern "C" {
 #define SPLUNK_SMALL_WHITE_ICON		"splunk1.gif"
 #define SPLUNK_SMALL_BLACK_ICON		"splunk2.gif"
 
+#define FIRST_PAGE_ICON			"b_first2.png"
+#define LAST_PAGE_ICON			"b_last2.png"
+#define NEXT_PAGE_ICON			"b_next2.png"
+#define PREVIOUS_PAGE_ICON		"b_prev2.png"
 
 
 	/************************** PLUGIN RETURN VALUES ****************************/
@@ -419,70 +423,67 @@ extern "C" {
 
 
 
-	/*************************** DATA STRUCTURES  *****************************/
+/*************************** DATA STRUCTURES  *****************************/
 
-	/* LIFO data structure */
-	typedef struct lifo_struct {
-		char *data;
-		struct lifo_struct *next;
-		} lifo;
+/* LIFO data structure */
+typedef struct lifo_struct {
+	char *data;
+	struct lifo_struct *next;
+	} lifo;
 
-	/******************************** FUNCTIONS *******************************/
+/******************************** FUNCTIONS *******************************/
 
-	void reset_cgi_vars(void);
-	void free_memory(void);
+void reset_cgi_vars(void);
+void free_memory(void);
 
-	char * get_cgi_config_location(void);				/* gets location of the CGI config file to read */
-	char * get_cmd_file_location(void);				/* gets location of external command file to write to */
+char *get_cgi_config_location(void);				/* gets location of the CGI config file to read */
+char *get_cmd_file_location(void);				/* gets location of external command file to write to */
 
-	int read_cgi_config_file(char *);
-	int read_main_config_file(char *);
-	int read_all_object_configuration_data(char *, int);
-	int read_all_status_data(char *, int);
+int read_cgi_config_file(char *);
+int read_main_config_file(char *);
+int read_all_object_configuration_data(char *, int);
+int read_all_status_data(char *, int);
 
-	char *unescape_newlines(char *);
-	void sanitize_plugin_output(char *);                            /* strips HTML and bad characters from plugin output */
-	void strip_html_brackets(char *);				/* strips > and < from string */
+char *unescape_newlines(char *);
+void sanitize_plugin_output(char *);                            /* strips HTML and bad characters from plugin output */
+void strip_html_brackets(char *);				/* strips > and < from string */
 
-	void get_time_string(time_t *, char *, int, int);			/* gets a date/time string */
-	void get_interval_time_string(double, char *, int);		/* gets a time string for an interval of time */
+void get_time_string(time_t *, char *, int, int);			/* gets a date/time string */
+void get_interval_time_string(double, char *, int);		/* gets a time string for an interval of time */
 
-	char * url_encode(char *);		        		/* encodes a string in proper URL format */
-	char * html_encode(char *, int);					/* encodes a string in HTML format (for what the user sees) */
-	char * escape_string(char *);					/* escape string for html form usage */
+char *url_encode(char *);		        		/* encodes a string in proper URL format */
+char *html_encode(char *, int);					/* encodes a string in HTML format (for what the user sees) */
+char *escape_string(char *);					/* escape string for html form usage */
 
-	void get_log_archive_to_use(int, char *, int);			/* determines the name of the log archive to use */
-	void determine_log_rotation_times(int);
-	int determine_archive_to_use_from_time(time_t);
+void get_log_archive_to_use(int, char *, int);			/* determines the name of the log archive to use */
+void determine_log_rotation_times(int);
+int determine_archive_to_use_from_time(time_t);
 
-	void print_extra_hostgroup_url(char *, char *);
-	void print_extra_servicegroup_url(char *, char *);
+void print_extra_hostgroup_url(char *, char *);
+void print_extra_servicegroup_url(char *, char *);
 
-	void display_info_table(char *, int, authdata *);
-	void display_nav_table(char *, int);
+void display_info_table(char *, int, authdata *);
+void display_nav_table(char *, int);
 
-	void display_splunk_host_url(host *);
-	void display_splunk_service_url(service *);
-	void display_splunk_generic_url(char *, int);
-	void strip_splunk_query_terms(char *);
+void display_splunk_host_url(host *);
+void display_splunk_service_url(service *);
+void display_splunk_generic_url(char *, int);
+void strip_splunk_query_terms(char *);
 
-	void include_ssi_files(char *, int);                            /* include user-defined SSI footers/headers */
-	void include_ssi_file(char *);                                  /* include user-defined SSI footer/header */
+void include_ssi_files(char *, int);                            /* include user-defined SSI footers/headers */
+void include_ssi_file(char *);                                  /* include user-defined SSI footer/header */
 
-	void cgi_config_file_error(char *);
-	void main_config_file_error(char *);
-	void object_data_error(void);
-	void status_data_error(void);
+void cgi_config_file_error(char *);
+void main_config_file_error(char *);
+void object_data_error(void);
+void status_data_error(void);
 
-	void display_context_help(char *);				/* displays context-sensitive help window */
+void display_context_help(char *);				/* displays context-sensitive help window */
 
-	int read_file_into_lifo(char *);				/* LIFO functions */
-	void free_lifo_memory(void);
-	int push_lifo(char *);
-	char *pop_lifo(void);
-
-#ifdef __cplusplus
-	}
-#endif
+int read_file_into_lifo(char *);				/* LIFO functions */
+void free_lifo_memory(void);
+int push_lifo(char *);
+char *pop_lifo(void);
 
+NAGIOS_END_DECL
 #endif
