@@ -668,6 +668,7 @@ int xsddefault_save_status_data(void) {
 		fprintf(fp, "\ttriggered_by=%lu\n", temp_downtime->triggered_by);
 		fprintf(fp, "\tfixed=%d\n", temp_downtime->fixed);
 		fprintf(fp, "\tduration=%lu\n", temp_downtime->duration);
+		fprintf(fp, "\tis_in_effect=%d\n", temp_downtime->is_in_effect);
 		fprintf(fp, "\tauthor=%s\n", temp_downtime->author);
 		fprintf(fp, "\tcomment=%s\n", temp_downtime->comment);
 		fprintf(fp, "\t}\n\n");
@@ -760,6 +761,7 @@ int xsddefault_read_status_data(char *config_file, int options) {
 	unsigned long triggered_by = 0;
 	unsigned long duration = 0L;
 	int x = 0;
+	int is_in_effect = FALSE;
 
 
 	/* initialize some vars */
@@ -815,26 +817,11 @@ int xsddefault_read_status_data(char *config_file, int options) {
 			data_type = XSDDEFAULT_PROGRAMSTATUS_DATA;
 		else if(!strcmp(input, "hoststatus {")) {
 			data_type = XSDDEFAULT_HOSTSTATUS_DATA;
-			temp_hoststatus = (hoststatus *)malloc(sizeof(hoststatus));
-			if(temp_hoststatus) {
-				temp_hoststatus->host_name = NULL;
-				temp_hoststatus->plugin_output = NULL;
-				temp_hoststatus->long_plugin_output = NULL;
-				temp_hoststatus->perf_data = NULL;
-				temp_hoststatus->check_options = 0;
-				}
+			temp_hoststatus = (hoststatus *)calloc(1, sizeof(hoststatus));
 			}
 		else if(!strcmp(input, "servicestatus {")) {
 			data_type = XSDDEFAULT_SERVICESTATUS_DATA;
-			temp_servicestatus = (servicestatus *)malloc(sizeof(servicestatus));
-			if(temp_servicestatus) {
-				temp_servicestatus->host_name = NULL;
-				temp_servicestatus->description = NULL;
-				temp_servicestatus->plugin_output = NULL;
-				temp_servicestatus->long_plugin_output = NULL;
-				temp_servicestatus->perf_data = NULL;
-				temp_servicestatus->check_options = 0;
-				}
+			temp_servicestatus = (servicestatus *)calloc(1, sizeof(servicestatus));
 			}
 		else if(!strcmp(input, "contactstatus {")) {
 			data_type = XSDDEFAULT_CONTACTSTATUS_DATA;
@@ -901,9 +888,9 @@ int xsddefault_read_status_data(char *config_file, int options) {
 
 					/* add the downtime */
 					if(data_type == XSDDEFAULT_HOSTDOWNTIME_DATA)
-						add_host_downtime(host_name, entry_time, author, comment_data, start_time, end_time, fixed, triggered_by, duration, downtime_id);
+						add_host_downtime(host_name, entry_time, author, comment_data, start_time, end_time, fixed, triggered_by, duration, downtime_id, is_in_effect);
 					else
-						add_service_downtime(host_name, service_description, entry_time, author, comment_data, start_time, end_time, fixed, triggered_by, duration, downtime_id);
+						add_service_downtime(host_name, service_description, entry_time, author, comment_data, start_time, end_time, fixed, triggered_by, duration, downtime_id, is_in_effect);
 
 					/* free temp memory */
 					my_free(host_name);
@@ -919,6 +906,7 @@ int xsddefault_read_status_data(char *config_file, int options) {
 					fixed = FALSE;
 					triggered_by = 0;
 					duration = 0L;
+					is_in_effect = FALSE;
 
 					break;
 
@@ -1279,6 +1267,8 @@ int xsddefault_read_status_data(char *config_file, int options) {
 						triggered_by = strtoul(val, NULL, 10);
 					else if(!strcmp(var, "duration"))
 						duration = strtoul(val, NULL, 10);
+					else if(!strcmp(var, "is_in_effect"))
+						is_in_effect = (atoi(val) > 0) ? TRUE : FALSE;
 					else if(!strcmp(var, "author"))
 						author = (char *)strdup(val);
 					else if(!strcmp(var, "comment"))
